/**
 * CompressedXMLFileReader.java
 * 
 */
package client;

import com.thoughtworks.xstream.XStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.GZIPInputStream;

/**
 * This class reads data-files that have been stored as gzipped XStream XML 
 * files.
 *
 * @author Martin Gross, Madeleine Theile
 *
 * @param <T> the data type stored in the file.
 */
public class CompressedXMLFileReader<T> {

    /**
     * The data read from the file. This is the whole file at once.
     */
    private T data;
    /**
     * The gzipped XML-file to read from.
     */
    private File file;
    /**
     * For reading the specific XML-format.
     */
    private XStream xstream;

    /**
     * Creates a reader for a given file.
     *
     * @param filename the name of the file to be read.
     */
    public CompressedXMLFileReader(String filename) {
        this(new File(filename));
    }

    /**
     * Creates a reader for a given file.
     *
     * @param file the file to be read.
     */
    public CompressedXMLFileReader(File file) {
        this.file = file;
        xstream = new XStream();
    }

    /**
     * Returns the name of the game stored in the file.
     *
     * @return the name of the game stored in the file.
     */
    public String getNameFromFile() {
        String name = this.file.getName();
        return name.substring(0, name.indexOf("["));
    }

    /**
     * Opens the file, reads it contents and returns it.
     *
     * @return the data stored in the file.
     */
    public T load() {
        try {
            InputStream input = new GZIPInputStream(new FileInputStream(file));
            data = (T) xstream.fromXML(input);
            input.close();
        } catch (FileNotFoundException ex) {
        } catch (IOException ex) {
        }
        return data;
    }
}
