/*
 * Buttons.java
 * 
 */
package client.gui;

import java.awt.event.ActionListener;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToolBar;

/**
 * An enumeration Buttons available in the Hive GUI.
 * 
 * @author Martin Gross, Jan-Philipp Kappmeier 
 */
enum Buttons {

    /**
     * A button used to quit the game.
     */
    Exit("Hive beenden", Images.EXIT, true, "exit"),
    /**
     * Starts an offline hive game.
     */
    StartOfflineGame("Offline-Spiel starten", Images.START_GAME, true, "offline"), 
    /**
     * A button used to connect to the server.
     */
    Connect("Zum Server verbinden", Images.CONNECT, true, "connect"), 
    /**
     * Starts, creates or joins an online hive game.
     */
    StartOnlineGame("Online-Spiel starten", Images.START_GAME, false, "online"), 
    /**
     * Zooms out a bit.
     */
    ZoomOut("Verkleinern", Images.MAGNIFY_OUT, false, "zoomOut"), 
    /**
     * Zooms in a bit.
     */
    ZoomIn("Vergrößern", Images.MAGNIFY_IN, false, "zoomIn"), 
    /**
     * Shows a hex grid.
     */
    HexGrid("Hex-Grid anzeigen", Images.HEX_GRID, false, "hex_grid"), 
    /**
     * Shows numbers in the hexagons.
     */
    HexNumbers("Hexagon-IDs anzeigen", Images.HEX_NUMBERS, false, "hex_numbers"), 
    /**
     * Loads a replay.
     */
    LoadReplay("Lade Replay", Images.REPLAY, true, "replay"), 
    /**
     * Resets the replay.
     */
    ToStart("Zum Anfang gehen", Images.START, false, "toStart"), 
    /**
     * One step backwards in the replay.
     */
    Backward("Einen Schritt zurück", Images.PREVIOUS, false, "backwards"), 
    /**
     * One step forward in the replay.
     */
    Forward("Einen Schritt weiter", Images.NEXT, false, "forwards"),
    /**
     * Moves the replay to the end.
     */
    ViewGame("Spiel beobachten", Images.START_GAME, false, "view"), 
    /**
     * Moves the replay to the end.
     */
    ToEnd("Zum Ende gehen", Images.END, false, "toEnd"), 
    /**
     * Initializes and afterwards destroys a SimpleAI that just computes one move
     */
    ComputeAIMove("Berechne einen Move der SimpleAI", Images.AIMOVE, true, "aiMove");
    private JButton button;

    private Buttons(String toolTip, ImageIcon image, boolean enabled, String actionCommand) {
        button = new JButton(image);
        button.setToolTipText(toolTip);
        button.setActionCommand(actionCommand);
        button.setContentAreaFilled(false);
        button.setOpaque(false);
        button.setFocusPainted(false);
        button.setBorderPainted(false);
        button.setEnabled(enabled);
    }

    /**
     * Adds an action listener to the button and <b>removes</b> action listener
     * that have been previously added!
     *
     * @param actionListener the new action listener
     */
    public void setActionListener(ActionListener actionListener) {
        for (ActionListener al : button.getActionListeners()) {
            button.removeActionListener(al);
        }
        button.addActionListener(actionListener);
    }

    public void setSelectedIcon(Icon selectedIcon) {
        button.setSelectedIcon(selectedIcon);
    }

    public void setSelected(boolean slectionValue) {
        button.setSelected(slectionValue);
    }

    public void setIcon(Icon defaultIcon) {
        button.setIcon(defaultIcon);
    }

    public void setEnabled(boolean b) {
        button.setEnabled(b);
    }

    public boolean isSelected() {
        return button.isSelected();
    }

    public void addToToolBar(JToolBar toolBar) {
        toolBar.add(button);
    }
}
