/**
 * Images.java Created: 09.02.2011, 17:20:32
 */
package client.gui;

import common.hive.PieceType;
import common.hive.PlayerColor;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * A class that loads images and gives static access to them. Both, icons and
 * images for the piece with the insects are loaded.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public class Images {

    final static int icon_size = 30;
    /**
     * Hash map that allows to access images by short names.
     */
    //public final static HashMap<String, BufferedImage> imageLinks = new HashMap<String, BufferedImage>();
    /**
     * Zoom in icon.
     */
    public final static ImageIcon MAGNIFY_IN = loadScaledImageIcon(icon_size, "Magnify+.png");
    /**
     * Zoom out icon.
     */
    public final static ImageIcon MAGNIFY_OUT = loadScaledImageIcon(icon_size, "Magnify-.png");
    /**
     * Zoom to the area icon.
     */
    public final static ImageIcon MAGNIFY_FIT = loadScaledImageIcon(icon_size, "Magnify=.png");
    /**
     * Connect icon.
     */
    public final static ImageIcon CONNECT = loadScaledImageIcon(icon_size, "Connect.png");
    /**
     * Disconnect icon.
     */
    public final static ImageIcon DISCONNECT = loadScaledImageIcon(icon_size, "Disconnect.png");
    /**
     * Join game icon.
     */
    public final static ImageIcon JOIN_GAME = loadScaledImageIcon(icon_size, "Join.png");
    /**
     * Start game icon.
     */
    public final static ImageIcon START_GAME = loadScaledImageIcon(icon_size, "StartGame.png");
    /**
     * Create game icon.
     */
    public final static ImageIcon CREATE_GAME = loadScaledImageIcon(icon_size, "CreateGame.png");
    /**
     * Exit hive.
     */
    public final static ImageIcon EXIT = loadScaledImageIcon(icon_size, "Exit.png");
    /**
     * Back to the beginning of a replay.
     */
    public final static ImageIcon START = loadScaledImageIcon(icon_size, "Start.png");
    /**
     * One step back in the replay.
     */
    public final static ImageIcon PREVIOUS = loadScaledImageIcon(icon_size, "Previous.png");
    /**
     * Go to next step in replay.
     */
    public final static ImageIcon NEXT = loadScaledImageIcon(icon_size, "Next.png");
    /**
     * Go to end in replay.
     */
    public final static ImageIcon END = loadScaledImageIcon(icon_size, "End.png");
    /**
     * Load a replay.
     */
    public final static ImageIcon REPLAY = loadScaledImageIcon(icon_size, "Replay.png");
    /**
     * Compute an AI move
     */
    public final static ImageIcon AIMOVE = loadScaledImageIcon(icon_size, "Denkblase.gif");
    /**
     * Compute an AI move
     */
    public final static ImageIcon HEX_NUMBERS = loadScaledImageIcon(icon_size, "hex_numbers.png");
    /**
     * Compute an AI move
     */
    public final static ImageIcon HEX_GRID = loadScaledImageIcon(icon_size, "hex_grid.png");

    public static final Image[] HEX = new Image[3];
    static {
        HEX[0] = new ImageIcon("images" + File.separator + "white_hex2.png").getImage();
        HEX[1] = new ImageIcon("images" + File.separator + "black_hex2.png").getImage();
        HEX[2] = new ImageIcon("images" + File.separator + "green_hex2.png").getImage();
    }

    public static final Image[] HEX_BIG = new Image[4];
    static {
        HEX_BIG[0] = new ImageIcon("images" + File.separator + "hex_white_big.png").getImage();
        HEX_BIG[1] = new ImageIcon("images" + File.separator + "hex_big_black.png").getImage();
        HEX_BIG[2] = new ImageIcon("images" + File.separator + "hex_big_green.png").getImage();
        HEX_BIG[3] = new ImageIcon("images" + File.separator + "hex_big_yellow.png").getImage();
    }

    private static ImageIcon loadScaledImageIcon(int max_size, int color, String filename, int copies, boolean selected) {
        if (copies <= 0) {
            throw new IllegalArgumentException("Copies must be at least 1");
        }

        Image unscaledInsect = new ImageIcon("images" + File.separator + filename).getImage();
        float ratio = unscaledInsect.getWidth(null) * 1.0f / unscaledInsect.getHeight(null);
        Image scaledInsect = new ImageIcon(unscaledInsect.getScaledInstance(Math.round(max_size * ratio), max_size, Image.SCALE_SMOOTH)).getImage();
        if (scaledInsect.getWidth(null) == -1 || scaledInsect.getHeight(null) == -1) {
            throw new IllegalArgumentException("images" + File.separator + filename + " not found.");
        }

        int OFFSET = scaledInsect.getWidth(null) / 3;

        BufferedImage bi = new BufferedImage(OFFSET * (copies - 1) + HEX[color].getWidth(null) + 2, HEX[color].getHeight(null), BufferedImage.TYPE_INT_ARGB);
        for (int i = 0; i < copies; ++i) {
            if (selected) {
                bi.getGraphics().drawImage(HEX[2], i * OFFSET, 0, null);
            } else {
                bi.getGraphics().drawImage(HEX[color], i * OFFSET, 0, null);
            }
            bi.getGraphics().drawImage(scaledInsect, i * OFFSET + (HEX[color].getWidth(null) - scaledInsect.getWidth(null)) / 2, (HEX[color].getHeight(null) - scaledInsect.getHeight(null)) / 2, null);
        }
        return new ImageIcon(bi);
    }

    /**
     * Loads an icon with the specified max size.
     *
     * @param max_size the maximum size
     * @param filename the filename for the icon
     * @return the loaded icon
     */
    private static ImageIcon loadScaledImageIcon(int max_size, String filename) {
        return loadScaledImageIcon(max_size, new ImageIcon("images" + File.separator + filename));
    }    
    
    private static ImageIcon loadScaledImageIcon(int max_size, ImageIcon icon) {
        double ratio = icon.getIconWidth() / (double) icon.getIconHeight();
        return new ImageIcon(icon.getImage().getScaledInstance((int) (max_size * ratio), max_size, Image.SCALE_SMOOTH));
    }

    /*
     * Loads the graphics for insects not yet in the game.
     */
    private static final Icon[][][][] ICONS;
    static {
        ICONS = new Icon[PieceType.values().length][PlayerColor.values().length][2][];
        for (PieceType type : PieceType.values()) {
            for (PlayerColor color : PlayerColor.values()) {
                ICONS[type.ordinal()][color.ordinal()][0] = new Icon[type.getNumberOfPieces() + 1];
                ICONS[type.ordinal()][color.ordinal()][1] = new Icon[type.getNumberOfPieces() + 1];
                for (int i = 1; i <= type.getNumberOfPieces(); i++) {
                    ICONS[type.ordinal()][color.ordinal()][0][i] = loadScaledImageIcon(icon_size, color.ordinal(), type.getLowerCaseName() + ".png", i, false);
                    ICONS[type.ordinal()][color.ordinal()][1][i] = loadScaledImageIcon(icon_size, color.ordinal(), type.getLowerCaseName() + ".png", i, true);
                }
            }
        }
    }

    public static Icon getIconFor(PieceType type, PlayerColor color) {
        return getIconFor(type, color, type.getNumberOfPieces(), false);
    }

    public static Icon getIconFor(PieceType type, PlayerColor color, int remainingPieces, boolean selected) {
        return ICONS[type.ordinal()][color.ordinal()][(selected)? 1 : 0][remainingPieces];
    }

    /*
     * Loads the graphics for insects in the game.
     */
    private static final BufferedImage[][] IMAGES;
    static {
        IMAGES = new BufferedImage[PieceType.values().length][PlayerColor.values().length];
        for (PieceType type : PieceType.values()) {
            Image tmp = new ImageIcon("images" + File.separator + type.getLowerCaseName() + "_white.png").getImage();
            BufferedImage img = new BufferedImage(tmp.getWidth(null), tmp.getHeight(null), BufferedImage.TYPE_INT_ARGB);
            img.getGraphics().drawImage(tmp, 0, 0, null);
            IMAGES[type.ordinal()][0] = img;
            tmp = new ImageIcon("images" + File.separator + type.getLowerCaseName() + "_black.png").getImage();
            img = new BufferedImage(tmp.getWidth(null), tmp.getHeight(null), BufferedImage.TYPE_INT_ARGB);
            img.getGraphics().drawImage(tmp, 0, 0, null);
            IMAGES[type.ordinal()][1] = img;
        }
    }

    public static BufferedImage getImageFor(PieceType type, PlayerColor color) {
        return IMAGES[type.ordinal()][color.ordinal()];
    }

    private static BufferedImage honey;
    static {
        Image tmp = new ImageIcon("images" + File.separator + "honeycomb5.png").getImage();
        BufferedImage bi = new BufferedImage(tmp.getWidth(null), tmp.getHeight(null), BufferedImage.TYPE_INT_ARGB);
        for (int i = 0; i < 1; ++i) {
            bi.getGraphics().drawImage(tmp, i * tmp.getWidth(null), 0, null);
        }
        honey = bi;
    }

    public static BufferedImage getHoneycomb() {
        return honey;
    }
}
