/**
 * JConnectDialog.java
 * 
 */
package client.gui;

import info.clearthought.layout.TableLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

/**
 * A swing dialog that allows to give information for connecting to a server.
 * A server, port and a user name and password can be inserted.
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
@SuppressWarnings( "serial" )
public class JConnectDialog extends JDialog {
	/** The status if the window is closed. */
	private int dialogResult = JOptionPane.CANCEL_OPTION;
	/** An input text field for server address. */
	private JTextField server;
	/** An input text field for the server port. */
	private JTextField port;
	/** An input text field for the user name. */
	private JTextField name;
	/** An input text field for the password. */
	private JPasswordField password;

	/**
	 * Creates the dialog window and initializes it. The position will be set to
	 * the center of the parent window.
	 * @param owner the owner window
	 * @param title the title of the window
	 */
	public JConnectDialog( Frame owner, String title ) {
		super( owner, title, true );
        setResizable(false);
		final int width = 500;
		final int height = 140;
		setSize( width, height );
		setLocation( owner.getX() + (owner.getWidth() - width) / 2, owner.getY() + (owner.getHeight() - height) / 2 );
		initComponents();
	}

	/**
	 * Initializes the swing components in the window.
	 */
	private void initComponents() {
		int space = 12;

		double[][] size = {
			{space, TableLayout.PREFERRED, space, TableLayout.FILL, space, TableLayout.PREFERRED, space, TableLayout.FILL, space},
			{space, TableLayout.PREFERRED, space, TableLayout.PREFERRED, space, TableLayout.PREFERRED, space }
		};

		JTexturedPanel mainPanel = new JTexturedPanel( new TableLayout( size ), Images.getHoneycomb());
		add( mainPanel );

		server = new JTextField( "127.0.0.1" );
		port = new JTextField( "25052" );
		name = new JTextField( "" );
		password = new JPasswordField( "" );

		JLabel lblServer = new JLabel( "Server:" , JLabel.CENTER );
        lblServer.setBackground(Color.LIGHT_GRAY.brighter());
        lblServer.setBorder(new BevelBorder(BevelBorder.LOWERED) {

            @Override
            public Insets getBorderInsets(Component c, Insets insets) {
                Insets i = super.getBorderInsets(c, insets);
                i.set(i.top + 1, i.left + 3, i.bottom + 1, i.right + 2);
                return i;
            }

        });
        lblServer.setOpaque(true);
		JLabel lblPort = new JLabel( "Port:" , JLabel.CENTER);
        lblPort.setBackground(Color.LIGHT_GRAY.brighter());
        lblPort.setBorder(new BevelBorder(BevelBorder.LOWERED) {

            @Override
            public Insets getBorderInsets(Component c, Insets insets) {
                Insets i = super.getBorderInsets(c, insets);
                i.set(i.top + 1, i.left + 3, i.bottom + 1, i.right + 2);
                return i;
            }

        });
        lblPort.setOpaque(true);
		JLabel lblName = new JLabel( "Name:"  , JLabel.CENTER);
        lblName.setBackground(Color.LIGHT_GRAY.brighter());
        lblName.setBorder(new BevelBorder(BevelBorder.LOWERED) {

            @Override
            public Insets getBorderInsets(Component c, Insets insets) {
                Insets i = super.getBorderInsets(c, insets);
                i.set(i.top + 1, i.left + 3, i.bottom + 1, i.right + 2);
                return i;
            }

        });
        lblName.setOpaque(true);
		JLabel lblPassword = new JLabel( "Passwort:"  , JLabel.CENTER);
        lblPassword.setBackground(Color.LIGHT_GRAY.brighter());
        lblPassword.setBorder(new BevelBorder(BevelBorder.LOWERED) {

            @Override
            public Insets getBorderInsets(Component c, Insets insets) {
                Insets i = super.getBorderInsets(c, insets);
                i.set(i.top + 1, i.left + 3, i.bottom + 1, i.right + 2);
                return i;
            }

        });
        lblPassword.setOpaque(true);

/*
		mainPanel.add( lblServer, "1,1" );
		mainPanel.add( lblPort, "7,1" );
		mainPanel.add( lblName, "1,3" );
		mainPanel.add( lblPassword, "1,5" );

		mainPanel.add( server, "3,1,5,1" );
		mainPanel.add( port, "9,1" );
		mainPanel.add( name, "3,3,9,3" );
		mainPanel.add( password, "3,5,9,5" );
        */

		mainPanel.add( lblServer, "1,1" );
		mainPanel.add( lblPort, "5,1" );
		mainPanel.add( lblName, "1,3" );
		mainPanel.add( lblPassword, "5,3" );

		mainPanel.add( server, "3,1" );
		mainPanel.add( port, "7,1" );
		mainPanel.add( name, "3,3" );
		mainPanel.add( password, "7,3" );

		JButton btnConnect = new JButton( "Verbinden" );
		btnConnect.setActionCommand( "connect" );
		btnConnect.addActionListener( acl );
		JButton btnCancel = new JButton( "Abbrechen" );
		btnCancel.setActionCommand( "cancel" );
		btnCancel.addActionListener( acl );

		mainPanel.add( btnCancel, "1,5,3,5" );
		mainPanel.add( btnConnect, "5,5,7,5" );
	}

	/**
	 * Returns the result state of the dialog. Can be {@link JOptionPane.CANCEL_OPTION}
	 * or {@link JOptionPane.OK_OPTION}.
	 * @return the result state of the dialog.
	 */
	public int getDialogResult() {
		return dialogResult;
	}

	/**
	 * Returns the inserted string for the server.
	 * @return the inserted string for the server
	 */
	public String getServer() {
		return server.getText();
	}

	/**
	 * Returns the inserted string for the name.
	 * @return the inserted string for the name
	 */
	@Override
	public String getName() {
		return name.getText();
	}

	/**
	 * Returns the inserted string for the port.
	 * @return the inserted string for the port
	 */
	public String getPort() {
		return port.getText();
	}

	/**
	 * Returns the inserted string for the password.
	 * @return the inserted string for the password
	 */
	public String getPassword() {
		// not very secure ;)
		return new String( password.getPassword() );
	}

	/**
	 * An action listener handling the events of the two buttons for canceling
	 * and connecting.
	 */
	private ActionListener acl = new ActionListener() {
		public void actionPerformed( ActionEvent e ) {
			if( e.getActionCommand().equals( "cancel" ) )
				dialogResult = JOptionPane.CANCEL_OPTION;
			else if( e.getActionCommand().equals( "connect" ) )
				dialogResult = JOptionPane.OK_OPTION;
			setVisible( false );
		}
	};
}
