/**
 * ServerCommand.java
 * Created: 23.02.2011, 15:19:08
 */
package client.network;

import java.io.IOException;
import java.io.PrintStream;
import common.hive.PieceType;
import common.hive.HexDirection;


/**
 * An implementation of the server command interface. A server handling that
 * can receive the commands is needed.
 * @author Jan-Philipp Kappmeier
 */
public class ServerCommand implements ServerCommandInterface {
	/** The out stream that sends the command to the server. */
	private PrintStream out;


	/**
	 * Initializes the server connection with a given server handler.
	 * @param handler the server handler receiving and sending messages
	 * @throws IOException if some error using the server connection occurred
	 */
	public ServerCommand( ServerHandler handler ) throws IOException {
		this.out = new PrintStream( handler.getOut(), true );
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void login( String name, String password ) {
		if( (name == null || name.isEmpty()) && (password == null || password.isEmpty()) )
			out.println( "LOGIN" );
		else
			out.println( "LOGIN | " + name + " | " + password );
	}
	

	/**
	 * {@inheritDoc}
	 */
	public void logout() {
		out.println( "LOGOUT" );
	}

	/**
	 * {@inheritDoc}
	 */
	public void chat( String channel, String message ) {
		out.println( "CHAT | " + channel + " | " + message );
		//System.out.println( "CHAT | " + channel + " | " + message );
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void chatJoinChannel( String channel, String oldChannel ) {
		out.println( "LEAVE_CHANNEL | " + oldChannel );
		//System.out.println( "Leave channel: " + oldChannel );
		out.println( "JOIN_CHANNEL | " + channel );
	}

	/**
	 * {@inheritDoc}
	 */
	public void createGame( String name ) {
		out.println( "CREATE_GAME | HIVE | " + name.trim() );
	}

	/**
	 * {@inheritDoc}
	 */
	public void joinGame( String name ) {
		out.println( "JOIN_GAME | " + name.trim() );
	}

	/**
	 * {@inheritDoc}
	 */
	public void startGame( String name ) {
		out.println( "START_GAME | " + name.trim() );
	}

	/**
	 * {@inheritDoc}
	 */
	public void hiveIntroduce( String gameName, PieceType pieceType ) {
		out.println( "GAME | " + gameName.trim() + " | WHITE_FIRST_PIECE | " + pieceType.ordinal() );
	}

	/**
	 * {@inheritDoc}
	 */
	public void hiveIntroduce( String gameName, PieceType pieceType, HexDirection dir ) {
		out.println( "GAME | " + gameName.trim() + " | BLACK_FIRST_PIECE | " + pieceType.ordinal() + " | " + dir.ordinal() );
	}

	/**
	 * {@inheritDoc}
	 */
	public void hiveIntroduce( String gameName, PieceType pieceType, int anchorPiece, HexDirection dir ) {
		//System.out.println( "GAME | " + gameName.trim() + " | INTRODUCE_PIECE | " + pieceType.ordinal() + " | " + anchorPiece + " | " + dir.ordinal() );
		out.println( "GAME | " + gameName.trim() + " | INTRODUCE_PIECE | " + pieceType.ordinal() + " | " + anchorPiece + " | " + dir.ordinal() );
	}

	/**
	 * {@inheritDoc}
	 */
	public void hiveMove( String gameName, int piece, int anchorPiece, HexDirection dir ) {
		//System.out.println( "GAME | " + gameName.trim() + " | MOVE_PIECE | " + piece + " | " + anchorPiece + " | " + dir.ordinal() );
		out.println( "GAME | " + gameName.trim() + " | MOVE_PIECE | " + piece + " | " + anchorPiece + " | " + dir.ordinal() );
	}

	@Override
	public void getGames() {
		out.println( "LIST_GAMES" );
	}
}
