/**
 * ServerHandler.java
 * Created: 23.02.2011, 15:07:01
 */
package client.network;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;
import common.hive.HexDirection;
import common.hive.PieceType;

/**
 *
 * @author Jan-Philipp Kappmeier
 */
public class ServerHandler extends Thread {
	private int port;
	private String server;
	private ServerListener callback;
	private Socket kkSocket = null;
	private boolean connectionClosed = false;
	/** Enables a verbose mode that prints all messages of the server. */
	private boolean verbose = false;
	
	public ServerHandler( String port, String server, ServerListener callback ) {
		this.port = Integer.parseInt( port );
		this.server = server;
		this.callback = callback;
	}
	
	public ServerHandler( String port, String server, ServerListener callback, boolean verbose  ) {
		this( port, server, callback );
		this.verbose = verbose;
	}	

	public void setConnectionClosed() {
		connectionClosed = true;
	}
	
	@Override
	public void run() {
		PrintWriter out = null;
		BufferedReader in = null;
		try {
			kkSocket = new Socket( server, port );
			out = new PrintWriter( kkSocket.getOutputStream(), true );
			in = new BufferedReader( new InputStreamReader( kkSocket.getInputStream() ) );
		} catch( UnknownHostException e ) {
			System.err.println( "Don't know about host: " + server );
			callback.failed( Failed.Connect );
			return;
			//System.exit( 1 );
		} catch( IOException e ) {
			System.err.println( "Couldn't get I/O for the connection to: " + server );
			callback.failed( Failed.Connect );
			return;
			//System.exit( 1 );
		}
		String fromServer;
		try {
			while( !connectionClosed && (fromServer = in.readLine()) != null ) {//hier steht die while-Schleife
				String[] parts = fromServer.split( "\\s*\\|\\s*" );
				if( verbose && (parts.length <= 2 || (!((parts[0].equals( "SUCCESS" ) ) && parts[1].equals( "LOGIN" ) ) ) ) )
					System.out.println( "Server: " + fromServer );
				if( fromServer.startsWith( "AWAITING_LOGIN" ) ) {
					callback.awaitLogin();
				} else if( fromServer.startsWith( "LOGGED_IN_AS" ) ) {
					callback.loginName( parts[1] );
				} else if( parts[0].equals( "CONNECTION_CLOSED" ) ) {
					callback.logout( null );
					connectionClosed = true;
				} else if( parts[0].equals( "EVENT" ) ) {
					// handle all events
					if( parts[1].equals( "LOGOUT" ) ) {
						
					} else if( parts[1].equals( "CHANNEL_JOINED" ) ) {
						callback.chatJoinChannel( parts[2], parts[3] );
					} else if( parts[1].equals( "CHANNEL_LEFT" ) ) {
						callback.chatLeaveChannel( parts[2], parts[3] );
					} else if( parts[1].equals( "GAME_CREATED" ) ) {
						callback.gameCreated( parts[2] );
					} else if( parts[1].equals( "GAME_STARTED" ) ) {
						callback.startGame( parts[2], parts[3] );
					} else if( parts[1].equals( "GAME_JOINED" ) ) {
						callback.joinGame( parts[2], parts[3] ); // a player has joined a game

					} else if(parts[1].equals( "GAME_RESULT" ) ) {
						int result;
						if( parts[3].equals( "WHITE_WON" ) )
							result = 0;
						else if( parts[3].equals( "BLACK_WON" ) )
							result = 1;
						else
							result = 2;
						callback.gameOver( parts[2], result );
					} else if( parts[1].equals( "GAME_OVER" ) ) {
//						if( parts.length == 3 )
//							callback.gameOverWin( parts[2].equals( "Black" ) ? PlayerColor.Black : PlayerColor.White );
//						else
//							callback.gameOverDraw();
					} else if( parts[1].equals( "GAME" ) ) {
						// we have a game message
						if( parts[4].equals( "WHITE_FIRST_PIECE" ) ) {
							callback.hiveWhiteFirst( parts[2], parts[3], PieceType.values()[Integer.parseInt( parts[5])] );
						} else if( parts[4].equals( "BLACK_FIRST_PIECE" ) ) {
							callback.hiveBlackFirst( parts[2], parts[3], PieceType.values()[Integer.parseInt( parts[5])], HexDirection.values()[Integer.parseInt( parts[6])] );
						} else if( parts[4].equals( "INTRODUCE_PIECE" ) ) {
							callback.hiveIntroducePiece( parts[2], parts[3], PieceType.values()[Integer.parseInt( parts[5])], Integer.parseInt( parts[6] ), HexDirection.values()[Integer.parseInt( parts[7] )] );
						} else if( parts[4].equals( "MOVE_PIECE" ) ) {
							callback.hiveMovePiece( parts[2], parts[3], Integer.parseInt( parts[5] ), Integer.parseInt( parts[6] ), HexDirection.values()[Integer.parseInt( parts[7] )] );
						} else if( parts[4].equals( "SET_WHITE_PLAYER" ) ) {

						}
					}
				} else if( parts[0].equals( "CHANNEL_INFO" ) ) {
					callback.chatInfo( parts[1] );
				} else if( parts[0].equals( "CHAT_MESSAGE" ) ) {
					callback.chatMessage( parts[1], parts[2], parts[3] );
				} else if( parts[0].equals( "GAMES" ) ) {
					callback.games( parts );
				} else if( parts[0].equals( "SUCCESS" ) ) {
					// perform all success-messages
					if( parts[1].equals( "JOIN_GAME" ) ) {
						callback.success( Success.JoinGame );
					} else if( parts[1].equals( "CREATE_GAME" ) ) {
						callback.success( Success.CreateGame );
					} else if( parts[1].equals( "START_GAME" ) ) {
						//callback.success( Success.StartGame );
					} else if( parts[1].equals( "LOGIN" ) ) {
						callback.success( Success.Login );
					} else if( parts[1].equals( "GAME" ) ) {
						// nothing yet
					} else {
					}
				}else if( parts[0].equals( "FAILED" ) ) {
					// perform all failed-messages
					if(parts.length == 3){
						if( parts[1].equals( "JOIN_GAME" ) )
							callback.failed( Failed.Join);//, parts[1] );
					}else{
						if( parts[3].equals( "INTRODUCE_PIECE" ) ) {
							callback.failed( Failed.Introduce );
						} else if( parts[3].equals( "CREATE_GAME" ) ) {
							callback.failed( Failed.Create );
						} else if( parts[3].equals( "START_GAME" ) ) {
							//callback.startGame( true, parts[2] ); // Nachricht kommt, wenn man als 1. Spieler das Spiel startet, damit erhaelt der Startspieler die Nachricht aber doppelt, siehe GAME_STARTED
						} else if( parts[3].equals( "LOGIN" ) ) {
							callback.failed( Failed.Login );
						}
					}
				}
			}
			out.close();
			in.close();
			kkSocket.close();
		} catch( IOException ex ) {
			Logger.getLogger( ServerHandler.class.getName() ).log( Level.SEVERE, null, ex );
			callback.logout( null );
		}
	}

	public OutputStream getOut() throws IOException {
		return kkSocket != null ? kkSocket.getOutputStream() : null;
	}

	public String getServer() {
		return server;
	}

	public int getPort() {
		return port;
	}
}
