/**
 * StackElement.java
 *
 */
package common;

/**
 * A simple, recursively constructed stack.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 * @param <T> the type of the data stored in the stack.
 */
public class StackElement<T> {

    /**
     * The stack element below this stack level.
     */
    private StackElement<T> below;    
    /**
     * The data stored in this element.
     */
    private T data;
    /**
     * The height of the stack.
     */
    private int height;

    /**
     * Creates a new stack of size 1.
     *
     * @param data the data stored in this element.
     */
    public StackElement(T data) {
        height = 1;
        this.data = data;
    }

    /**
     * Adds a new level to an already existing stack.
     *
     * @param below the stack below this element.
     * @param data the data stored in the stack
     */
    public StackElement(StackElement<T> below, T data) {
        this.below = below;
        this.data = data;
        height = below.getHeight() + 1;
    }

    /**
     * Returns the element below this level on the stack.
     *
     * @return the element below this level on the stack
     */
    public StackElement<T> getBelow() {
        return below;
    }

    /**
     * Returns the data stored in this element.
     *
     * @return the data stored in this element.
     */
    public T getData() {
        return data;
    }

    /**
     * Returns the height of the stack.
     *
     * @return the height of the stack.
     */
    public int getHeight() {
        return height;
    }
}
