/**
 * AbstractHiveModel.java
 * 
 */
package common.interfaces;

import common.notification.PieceNotification;
import java.util.Observable;

/**
 * A base class for the hive game model that realized a framework for adding 
 * observers and notifying them. It does so by extending {@link Observable} thus
 * implementing Observable/Observer design pattern. The observers should be 
 * informed whenever the hive model is changed, that is when a move happened or 
 * when a new piece is introduced. See common.notification.
 * 
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public abstract class AbstractHiveModel extends Observable implements HiveInterface {

	/**
	 * Notifies the listening observers. The only possible message that can be
	 * handled is of the type {@link PieceNotification}. If any other message
	 * is passed an exception is thrown.
	 * @param message the {@link PieceNotification} message.
	 * @throws IllegalArgumentException if the passed argument is not of the type {@link PieceNotification}
	 */
	@Override
	public void notifyObservers( Object message ) throws IllegalArgumentException {
		if( !(message instanceof PieceNotification) )
			throw new IllegalArgumentException();
		notifyObservers( (PieceNotification)message );
	}

	/**
	 * Notifies the listening observers that a new piece operation occurred.
	 * @param message the {@link PieceNotification} message.
	 *
	 */
	public void notifyObservers( PieceNotification message ) {
		setChanged();
		super.notifyObservers( message );
	}
}
