/*
 * ClientHiveInterface.java
 * 
 */
package common.interfaces;

import common.hive.HexDirection;
import common.hive.PieceType;

/**
 * This class extends the hive interface for client-server purposes. The
 * inherited introduce and move methods should be sending moves from the client
 * to the server, and the new local variants of these methods are called by the
 * client to update the model of with moves send by the server.
 *
 * @author Martin Gross
 */
public interface ClientHiveInterface extends HiveInterface {

    /**
     * Returns the name of the game on the server.
     *
     * @return the name of the game on the server.
     */
    String getGameName();

    /**
     * Sends the information that the first piece was introduced to the local
     * model.
     *
     * @param pieceType the piece type of the first piece
     */
    void introduceLocal(PieceType pieceType);

    /**
     * Sends the information that a piece was introduced (except the first one)
     * to the local model.
     *
     * @param pieceType the type of the introduced piece
     * @param anchorPiece a piece that is next to the new piece
     * @param hexDirection the direction in which the new piece is introduced,
     * relatively to the anchor
     */
    void introduceLocal(PieceType pieceType, int anchorPiece, HexDirection hexDirection);

    /**
     * Sends the information that a piece has been moved to the local model.
     *
     * @param piece the piece hat has been moved
     * @param targetPiece a piece next to the target position
     * @param hexDirection the direction in which the piece is placed,
     * relatively to the target piece
     */
    void moveLocal(int piece, int targetPiece, HexDirection hexDirection);
}
