/**
 * DataContainer.java
 *
 */
package common.interfaces;

/**
 * An interface defining a data container. This is an object containing some
 * type of (generic) data. Each of the container has an index and can move the
 * data to another data container of the same type.
 *
 * @param <T> the generic type of the data stored in the container.
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public interface DataContainer<T> {

    /**
     * Check method to test if the container contains some data.
     *
     * @return {@code true} if the container contains some useful data (i.e. 
     * not null), {@code false} otherwise.
     */
    boolean containsData();

    /**
     * Returns the data in this container.
     * 
     * @return the data in this container.
     */
    T getData();    
    
    /**
     * Returns the index of the container.
     *
     * @return the index of the container.
     */
    int getIndex();

    /**
     * Transfers the data from one container to another. If data can be
     * overridden must be defined by the implementing class.
     *
     * @param target the target data container.
     */
    void moveDataTo(DataContainer<T> target);

    /**
     * Resets the data in container (i.e. sets the data to null).
     */
    void resetData();

    /**
     * Overwrites the data in this container with new data.
     * 
     * @param t the new data to write.
     */
    void setNewData(T t);
}
