/**
 * MoveNotification.java
 * Created: 11.03.2011, 15:00:10
 */
package common.notification;

import common.hive.HexDirection;

/**
 * A piece notification used if a piece is moved.
 * Stores the information about the piece underneath if a bug is moved on top.
 * @author Jan-Philipp Kappmeier
 */
public class MoveNotification extends PieceNotification {
	/** The index of the piece that is already on the targe field, if any. (Used for stackable pieces). */
	int below;

	/**
	 * <p><b>Note: the parameter below is used, if the bug was on top of another piece.</b></p>
	 * @param pieceIndex the index of the piece that performs an action
	 * @param direction the direction of the target field starting at the anchor field
	 * @param anchorPiece the index on the piece on the anchor field.
	 * @param below the index of the piece that occupies the field below the bug. should be -1 if the moved piece was on the lowest area.
	 */
	public MoveNotification( int pieceIndex, HexDirection direction, int anchorPiece, int below ) {
		super( pieceIndex, direction, anchorPiece );
		this.below = below;
	}

	/**
	 * Returns the index of the piece that is below the piece that is moved. If the
	 * moved piece is not on top of other pieces, -1 should be returned.
	 * @return the index of the piece on top of the target field or -1.
	 */
	public int getBelow() {
		return below;
	}
}
